package com.blockchain.server.yyyf.service.impl;

import com.blockchain.server.yyyf.dto.TransactResultDto;
import com.blockchain.server.yyyf.entity.YyyfMoney;
import com.blockchain.server.yyyf.enums.YyyfEnums;
import com.blockchain.server.yyyf.exceprion.YyyfException;
import com.blockchain.server.yyyf.mapper.YyyfMoneyMapper;
import com.blockchain.server.yyyf.service.YyyfMoneyService;
import lombok.SneakyThrows;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.util.Date;

/**
 * @author zijie
 * @version 1.0
 * @className YyyfMoneyServiceImpl
 * @description
 * @date 2020-05-26 21:36
 */
@Transactional(rollbackFor = Exception.class)
@Service
public class YyyfMoneyServiceImpl implements YyyfMoneyService {

    private Logger logger = LoggerFactory.getLogger(YyyfMoneyServiceImpl.class);

    @Autowired
    private YyyfMoneyMapper yyyfMoneyMapper;

 /*   @Autowired
    private CurrencyFeign currencyFeign;*/

    @SneakyThrows
    @Override
    public TransactResultDto transact(String assessUserId, String out, String in, BigDecimal amount) {

        YyyfMoney yyyfMoney = this.yyyfMoneyMapper.selectByPrimaryKey(assessUserId);
        BigDecimal outOriginal = null;
        BigDecimal inOriginal = null;
        Class<? extends YyyfMoney> aClass = yyyfMoney.getClass();

        String outField = out.substring(0, 1).toUpperCase().concat(out.substring(1));
        String inField = in.substring(0, 1).toUpperCase().concat(in.substring(1));

        Method outGetMethod = aClass.getMethod("get".concat(outField));
        //转出的原始金额
        outOriginal = (BigDecimal) outGetMethod.invoke(yyyfMoney);

        Method inGetMethod = aClass.getMethod("get".concat(inField));
        //转入的原始金额
        inOriginal = (BigDecimal) inGetMethod.invoke(yyyfMoney);

        if (outOriginal.compareTo(amount) == -1) {
            throw new YyyfException(YyyfEnums.MONEY_LOW);
        }
        //转出币种后的余额
        BigDecimal outSurplus = outOriginal.subtract(amount);
        //转入币种后的余额
        BigDecimal inSurplus = inOriginal.add(amount);

        Method outSetMethod = aClass.getMethod("set".concat(outField), BigDecimal.class);
        Method inSetMethod = aClass.getMethod("set".concat(inField), BigDecimal.class);

        outSetMethod.invoke(yyyfMoney, outSurplus);
        inSetMethod.invoke(yyyfMoney, inSurplus);

        this.yyyfMoneyMapper.updateByPrimaryKeySelective(yyyfMoney);

        TransactResultDto transactResultDto = new TransactResultDto();

        transactResultDto.setAmount(amount);
        transactResultDto.setOut(out);
        transactResultDto.setOutOriginal(outOriginal);
        transactResultDto.setOutSurplus(outSurplus);
        transactResultDto.setIn(in);
        transactResultDto.setInOriginal(inOriginal);
        transactResultDto.setInSurplus(inSurplus);


        return transactResultDto;
    }

    @Override
    public void insert(YyyfMoney yyyfMoney) {
        this.yyyfMoneyMapper.insert(yyyfMoney);
    }

    @Override
    public void update(YyyfMoney yyyfMoney) {
        this.yyyfMoneyMapper.updateByPrimaryKeySelective(yyyfMoney);
    }

    @Override
    public boolean resetWallet(String assessUserId, String coinName, BigDecimal balance) {

        YyyfMoney yyyfMoney = this.yyyfMoneyMapper.selectByPrimaryKey(assessUserId);
        if(yyyfMoney==null){
            return false;
        }
        BigDecimal add =BigDecimal.ZERO;
        coinName=coinName.toLowerCase();

        switch (coinName) {
            case"zfb":{
                add = yyyfMoney.getAliPay().add(balance);
                yyyfMoney.setAliPay(add);
                break;
            }
            case"wx":{
                add= yyyfMoney.getWxPay().add(balance);
                yyyfMoney.setWxPay(add);
                break;
            }
            case"bank":{
                add = yyyfMoney.getBankCard().add(balance);
                yyyfMoney.setBankCard(add);
                break;
            }
            case "btc": {
                add = yyyfMoney.getBtc().add(balance);
                yyyfMoney.setBtc(add);
                break;
            }
            case "usdt": {
                add = yyyfMoney.getUsdt().add(balance);
                yyyfMoney.setUsdt(add);

                break;
            }
            case "eth": {
                add = yyyfMoney.getEth().add(balance);
                yyyfMoney.setEth(add);
                break;
            }
            case "eos": {
                add = yyyfMoney.getEos().add(balance);
                yyyfMoney.setEos(add);
                break;
            }
            default:
                add=BigDecimal.ZERO.subtract(BigDecimal.ONE);
        }
        if(add.compareTo(BigDecimal.ZERO)==-1){
            return false;
        }else{
            yyyfMoney.setUpdateTime(new Date());
            this.yyyfMoneyMapper.updateByPrimaryKeySelective(yyyfMoney);
        }
        return true;
    }

}
